@@ -6,11 +6,14 @@
 # This work is licensed under the GNU GPLv2 or later.
 # See the COPYING file in the top-level directory.
 
-import logging
+import os
+
+from ..logger import log
 
 from .. import diskbackend
-from .. import util
-from .device import Device
+from .. import progress
+from .. import xmlutil
+from .device import Device, DeviceAddress, DeviceSeclabel
 from ..xmlbuilder import XMLBuilder, XMLChildProperty, XMLProperty
 
 
@@ -28,22 +31,17 @@ def _qemu_sanitize_drvtype(phystype, fmt):
 
 
 class _Host(XMLBuilder):
-    _XML_PROP_ORDER = ["name", "port"]
+    _XML_PROP_ORDER = ["name", "port", "transport", "socket"]
     XML_NAME = "host"
 
     name = XMLProperty("./@name")
     port = XMLProperty("./@port", is_int=True)
+    transport = XMLProperty("./@transport")
+    socket = XMLProperty("./@socket")
 
 
-class _DiskSeclabel(XMLBuilder):
-    """
-    This is for disk source <seclabel>. It's similar to a domain
-    <seclabel> but has fewer options
-    """
-    XML_NAME = "seclabel"
-    model = XMLProperty("./@model")
-    relabel = XMLProperty("./@relabel", is_yesno=True)
-    label = XMLProperty("./label")
+class _DiskSourceAddress(DeviceAddress):
+    pass
 
 
 class DeviceDisk(Device):
@@ -83,56 +81,7 @@ class DeviceDisk(Device):
     TYPE_NETWORK = "network"
 
     IO_MODE_NATIVE = "native"
-    IO_MODE_THREADS = "threads"
-    IO_MODES = [IO_MODE_NATIVE, IO_MODE_THREADS]
 
-    @staticmethod
-    def get_old_recommended_buses(guest):
-        ret = []
-        if guest.os.is_hvm() or guest.conn.is_test():
-            if not guest.os.is_q35():
-                ret.append("ide")
-            ret.append("sata")
-            ret.append("fdc")
-            ret.append("scsi")
-            ret.append("usb")
-
-            if guest.type in ["qemu", "kvm", "test"]:
-                ret.append("sd")
-                ret.append("virtio")
-                if "scsi" not in ret:
-                    ret.append("scsi")
-
-        if guest.conn.is_xen() or guest.conn.is_test():
-            ret.append("xen")
-
-        return ret
-
-    @staticmethod
-    def get_recommended_buses(guest, domcaps, devtype):
-        # try to get supported disk bus types from domain capabilities
-        if "bus" in domcaps.devices.disk.enum_names():
-            buses = domcaps.devices.disk.get_enum("bus").get_values()
-        else:
-            buses = DeviceDisk.get_old_recommended_buses(guest)
-
-        bus_map = {
-            "disk": ["ide", "sata", "scsi", "sd", "usb", "virtio", "xen"],
-            "floppy": ["fdc"],
-            "cdrom": ["ide", "sata", "scsi"],
-            "lun": ["scsi"],
-        }
-        return [bus for bus in buses if bus in bus_map.get(devtype, [])]
-
-    @staticmethod
-    def pretty_disk_bus(bus):
-        if bus in ["ide", "sata", "scsi", "usb", "sd"]:
-            return bus.upper()
-        if bus in ["xen"]:
-            return bus.capitalize()
-        if bus == "virtio":
-            return "VirtIO"
-        return bus
 
     @staticmethod
     def path_definitely_exists(conn, path):
@@ -169,20 +118,22 @@ class DeviceDisk(Device):
         searchdata = SearchData()
         if path is None:
             return searchdata
+
         if conn.is_remote():
             return searchdata
-        if not conn.is_qemu_system():
+        if not conn.is_qemu_privileged():
             return searchdata
         if diskbackend.path_is_url(path):
             return searchdata
         if diskbackend.path_is_network_vol(conn, path):
             return searchdata
+        path = os.path.abspath(path)
 
         user, uid = conn.caps.host.get_qemu_baselabel()
         if not user:
             return searchdata
-        if uid == 0:
-            return searchdata
+        if uid == 0 and not xmlutil.in_testsuite():
+            return searchdata  # pragma: no cover
 
         searchdata.user = user
         searchdata.uid = uid
@@ -191,13 +142,12 @@ class DeviceDisk(Device):
         return searchdata
 
     @staticmethod
-    def fix_path_search(conn, searchdata):
+    def fix_path_search(searchdata):
         """
         Try to fix any permission problems found by check_path_search
 
         :returns: Return a dictionary of entries {broken path : error msg}
         """
-        ignore = conn
         errdict = diskbackend.set_dirs_searchable(
                 searchdata.fixlist, searchdata.user)
         return errdict
@@ -225,7 +175,7 @@ class DeviceDisk(Device):
         while backpath in volmap:
             vol = volmap[backpath]
             if vol in vols:
-                break
+                break  # pragma: no cover
             backpath = vol.target_path
             vols.append(backpath)
 
@@ -278,7 +228,7 @@ class DeviceDisk(Device):
                 "path '%s'. Use libvirt APIs to manage the parent directory "
                 "as a pool first.") % volname)
 
-        logging.debug("Creating volume '%s' on pool '%s'",
+        log.debug("Creating volume '%s' on pool '%s'",
                       volname, poolobj.name())
 
         cap = (size * 1024 * 1024 * 1024)
@@ -296,7 +246,7 @@ class DeviceDisk(Device):
         volinst.backing_format = backing_format
 
         if fmt:
-            if not volinst.supports_property("format"):
+            if not volinst.supports_format():
                 raise ValueError(_("Format attribute not supported for this "
                                    "volume type"))
             volinst.format = fmt
@@ -317,15 +267,10 @@ class DeviceDisk(Device):
             num -= amt
             digits.insert(0, amt)
 
-        seen_valid = False
         gen_t = ""
         for digit in digits:
             if digit == 0:
-                if not seen_valid:
-                    continue
-                digit = 1
-
-            seen_valid = True
+                continue
             gen_t += "%c" % (ord('a') + digit - 1)
 
         return gen_t
@@ -349,15 +294,16 @@ class DeviceDisk(Device):
 
 
     _XML_PROP_ORDER = [
-        "_type", "_device", "snapshot_policy",
+        "_xmltype", "_device", "snapshot_policy",
         "driver_name", "driver_type",
         "driver_cache", "driver_discard", "driver_detect_zeroes",
         "driver_io", "error_policy",
-        "_source_file", "_source_dev", "_source_dir",
+        "source_file", "source_dev", "source_dir",
         "auth_username", "auth_secret_type", "auth_secret_uuid",
         "source_volume", "source_pool", "source_protocol", "source_name",
         "source_host_name", "source_host_port",
         "source_host_transport", "source_host_socket",
+        "source_type", "source_managed", "source_namespace", "source_address",
         "target", "bus",
     ]
 
@@ -365,27 +311,26 @@ class DeviceDisk(Device):
         Device.__init__(self, *args, **kwargs)
 
         self._source_volume_err = None
-        self._storage_backend = None
         self.storage_was_created = False
 
+        self._storage_backend = diskbackend.StorageBackendStub(
+            self.conn, self._get_xmlpath(), self._xmltype, self.driver_type)
+
 
     #############################
     # Public property-esque API #
     #############################
 
     def _get_path(self):
-        if not self._storage_backend:
-            xmlpath = self._get_xmlpath()
-            if xmlpath:
-                return xmlpath
-
-            self._set_default_storage_backend()
+        if (self._storage_backend.is_stub() and not
+            self._storage_backend.get_path()):
+            self._resolve_storage_backend()
         return self._storage_backend.get_path()
     def _set_path(self, newpath):
-        if (self._storage_backend and
-            self._storage_backend.will_create_storage()):
-            raise ValueError(_("Can't change disk path if storage creation info "
-                               "has been set."))
+        if self._storage_backend.will_create_storage():
+            raise xmlutil.DevError(
+                    "Can't change disk path if storage creation info "
+                    "has been set.")
 
         # User explicitly changed 'path', so try to lookup its storage
         # object since we may need it
@@ -395,16 +340,24 @@ class DeviceDisk(Device):
         self._set_xmlpath(self.path)
     path = property(_get_path, _set_path)
 
+    def set_backend_for_existing_path(self):
+        # This is an entry point for parsexml Disk instances to request
+        # a _storage_backend to be initialized from the XML path. That
+        # will cause validate() to actually validate the path exists.
+        # We need this so addhw XML editing will still validate the disk path
+        if self._storage_backend.is_stub():
+            self._resolve_storage_backend()
+
     def set_vol_object(self, vol_object, parent_pool):
-        logging.debug("disk.set_vol_object: volxml=\n%s",
+        log.debug("disk.set_vol_object: volxml=\n%s",
             vol_object.XMLDesc(0))
-        logging.debug("disk.set_vol_object: poolxml=\n%s",
+        log.debug("disk.set_vol_object: poolxml=\n%s",
             parent_pool.XMLDesc(0))
         self._change_backend(None, vol_object, parent_pool)
         self._set_xmlpath(self.path)
 
     def set_vol_install(self, vol_install):
-        logging.debug("disk.set_vol_install: name=%s poolxml=\n%s",
+        log.debug("disk.set_vol_install: name=%s poolxml=\n%s",
             vol_install.name, vol_install.pool.XMLDesc(0))
         self._storage_backend = diskbackend.ManagedStorageCreator(
             self.conn, vol_install)
@@ -413,14 +366,9 @@ class DeviceDisk(Device):
     def get_vol_object(self):
         return self._storage_backend.get_vol_object()
     def get_vol_install(self):
-        if not self._storage_backend:
-            return None
         return self._storage_backend.get_vol_install()
     def get_parent_pool(self):
-        if self.get_vol_install():
-            return self.get_vol_install().pool
         return self._storage_backend.get_parent_pool()
-
     def get_size(self):
         return self._storage_backend.get_size()
 
@@ -438,8 +386,7 @@ class DeviceDisk(Device):
         # If type block, use name=phy. Otherwise do the same as qemu
         if self.conn.is_xen() and self.type == self.TYPE_BLOCK:
             return self.DRIVER_NAME_PHY
-        if self.conn.check_support(
-                self.conn.SUPPORT_CONN_DISK_DRIVER_NAME_QEMU):
+        if self.conn.support.conn_disk_driver_name_qemu():
             return self.DRIVER_NAME_QEMU
         return None
 
@@ -464,9 +411,9 @@ class DeviceDisk(Device):
     # XML source media handling #
     #############################
 
-    _source_file = XMLProperty("./source/@file")
-    _source_dev = XMLProperty("./source/@dev")
-    _source_dir = XMLProperty("./source/@dir")
+    source_file = XMLProperty("./source/@file")
+    source_dev = XMLProperty("./source/@dev")
+    source_dir = XMLProperty("./source/@dir")
 
     source_pool = XMLProperty("./source/@pool")
     source_volume = XMLProperty("./source/@volume")
@@ -485,6 +432,12 @@ class DeviceDisk(Device):
     source_host_transport = XMLProperty("./source/host/@transport")
     source_host_socket = XMLProperty("./source/host/@socket")
 
+    source_type = XMLProperty("./source/@type")
+    source_managed = XMLProperty("./source/@managed", is_yesno=True)
+    source_namespace = XMLProperty("./source/@namespace", is_int=True)
+    source_address = XMLChildProperty(
+            _DiskSourceAddress, is_single=True, relative_xpath="./source")
+
     def _set_source_network_from_url(self, uri):
         from ..uri import URI
         uriobj = URI(uri)
@@ -506,9 +459,12 @@ class DeviceDisk(Device):
                     self.source_name = self.source_name[1:]
 
     def _set_source_network_from_storage(self, volxml, poolxml):
-        self.source_protocol = poolxml.type
-        logging.debug("disk.set_vol_object: poolxml=\n%s",
-                      dir(poolxml))
+        is_iscsi_direct = poolxml.type == "iscsi-direct"
+        protocol = poolxml.type
+        if is_iscsi_direct:
+            protocol = "iscsi"
+
+        self.source_protocol = protocol
         if poolxml.auth_type:
             self.auth_username = poolxml.auth_username
             self.auth_secret_type = poolxml.auth_type
@@ -522,14 +478,22 @@ class DeviceDisk(Device):
                 obj.port = host.port
 
         path = ""
-        if poolxml.source_name:
-            path += poolxml.source_name
-            if poolxml.source_path:
-                path += poolxml.source_path
-            if not path.endswith('/'):
-                path += "/"
-        path += volxml.name
-        self.source_name = path
+        if is_iscsi_direct:
+            # Vol path is like this:
+            # ip-10.66.144.87:3260-iscsi-iqn.2017-12.com.virttest:emulated-iscsi-noauth.target2-lun-1
+            # Always seems to have -iscsi- embedded in it
+            if "-iscsi-iqn." in volxml.target_path:
+                path = volxml.target_path.split("-iscsi-", 1)[-1]
+        else:
+            if poolxml.source_name:
+                path += poolxml.source_name
+                if poolxml.source_path:
+                    path += poolxml.source_path
+                if not path.endswith('/'):
+                    path += "/"
+            path += volxml.name
+        self.source_name = path or None
+
         self.type = "network"
 
     def _set_network_source_from_backend(self):
@@ -542,7 +506,7 @@ class DeviceDisk(Device):
             self._set_source_network_from_url(self._storage_backend.get_path())
 
     def _build_url_from_network_source(self):
-        ret = self.source_protocol
+        ret = self.source_protocol or "unknown"
         if self.source_host_transport:
             ret += "+%s" % self.source_host_transport
         ret += "://"
@@ -563,7 +527,7 @@ class DeviceDisk(Device):
     def _get_default_type(self):
         if self.source_pool or self.source_volume:
             return DeviceDisk.TYPE_VOLUME
-        if self._storage_backend:
+        if not self._storage_backend.is_stub():
             return self._storage_backend.get_dev_type()
         if self.source_protocol:
             return DeviceDisk.TYPE_NETWORK
@@ -573,9 +537,9 @@ class DeviceDisk(Device):
         """
         Unset all XML properties that describe the actual source media
         """
-        self._source_file = None
-        self._source_dev = None
-        self._source_dir = None
+        self.source_file = None
+        self.source_dev = None
+        self.source_dir = None
         self.source_volume = None
         self.source_pool = None
         self.source_name = None
@@ -588,11 +552,11 @@ class DeviceDisk(Device):
     def _disk_type_to_object_prop_name(self):
         disk_type = self.type
         if disk_type == DeviceDisk.TYPE_BLOCK:
-            return "_source_dev"
+            return "source_dev"
         elif disk_type == DeviceDisk.TYPE_DIR:
-            return "_source_dir"
+            return "source_dir"
         elif disk_type == DeviceDisk.TYPE_FILE:
-            return "_source_file"
+            return "source_file"
         return None
 
 
@@ -600,12 +564,12 @@ class DeviceDisk(Device):
     # they don't have any special properties aside from needing to match
     # 'type' value with the source property used.
     def _get_xmlpath(self):
-        if self._source_file:
-            return self._source_file
-        if self._source_dev:
-            return self._source_dev
-        if self._source_dir:
-            return self._source_dir
+        if self.source_file:
+            return self.source_file
+        if self.source_dev:
+            return self.source_dev
+        if self.source_dir:
+            return self.source_dir
         return None
     def _set_xmlpath(self, val):
         self._clear_source_xml()
@@ -627,13 +591,13 @@ class DeviceDisk(Device):
     # type, device, driver_name, driver_type handling
     # These are all weirdly intertwined so require some special handling
     def _get_type(self):
-        if self._type:
-            return self._type
+        if self._xmltype:
+            return self._xmltype
         return self._get_default_type()
     def _set_type(self, val):
-        self._type = val
+        self._xmltype = val
     type = property(_get_type, _set_type)
-    _type = XMLProperty("./@type")
+    _xmltype = XMLProperty("./@type")
 
     def _get_device(self):
         if self._device:
@@ -652,6 +616,7 @@ class DeviceDisk(Device):
     driver_copy_on_read = XMLProperty("./driver/@copy_on_read", is_onoff=True)
 
     sgio = XMLProperty("./@sgio")
+    rawio = XMLProperty("./@rawio")
 
     bus = XMLProperty("./target/@bus")
     target = XMLProperty("./target/@dev")
@@ -663,9 +628,11 @@ class DeviceDisk(Device):
     driver_discard = XMLProperty("./driver/@discard")
     driver_detect_zeroes = XMLProperty("./driver/@detect_zeroes")
     driver_io = XMLProperty("./driver/@io")
+    driver_iothread = XMLProperty("./driver/@iothread", is_int=True)
 
     error_policy = XMLProperty("./driver/@error_policy")
     serial = XMLProperty("./serial")
+    wwn = XMLProperty("./wwn")
     startup_policy = XMLProperty("./source/@startupPolicy")
     logical_block_size = XMLProperty("./blockio/@logical_block_size")
     physical_block_size = XMLProperty("./blockio/@physical_block_size")
@@ -677,7 +644,7 @@ class DeviceDisk(Device):
     iotune_wbs = XMLProperty("./iotune/write_bytes_sec", is_int=True)
     iotune_wis = XMLProperty("./iotune/write_iops_sec", is_int=True)
 
-    seclabels = XMLChildProperty(_DiskSeclabel, relative_xpath="./source")
+    seclabels = XMLChildProperty(DeviceSeclabel, relative_xpath="./source")
 
     geometry_cyls = XMLProperty("./geometry/@cyls", is_int=True)
     geometry_heads = XMLProperty("./geometry/@heads", is_int=True)
@@ -694,7 +661,7 @@ class DeviceDisk(Device):
     # Validation assistance methods #
     #################################
 
-    def _set_default_storage_backend(self):
+    def _resolve_storage_backend(self):
         path = None
         vol_object = None
         parent_pool = None
@@ -706,22 +673,23 @@ class DeviceDisk(Device):
             path = self._build_url_from_network_source()
 
         if typ == DeviceDisk.TYPE_VOLUME:
-            conn = self.conn
-            if "weakref" in str(type(conn)):
-                conn = conn()
-
             try:
-                parent_pool = conn.storagePoolLookupByName(self.source_pool)
+                parent_pool = self.conn.storagePoolLookupByName(
+                    self.source_pool)
                 vol_object = parent_pool.storageVolLookupByName(
                     self.source_volume)
             except Exception as e:
                 self._source_volume_err = str(e)
-                logging.debug("Error fetching source pool=%s vol=%s",
+                log.debug("Error fetching source pool=%s vol=%s",
                     self.source_pool, self.source_volume, exc_info=True)
 
         if vol_object is None and path is None:
             path = self._get_xmlpath()
 
+        if path and not vol_object and not parent_pool:
+            (vol_object, parent_pool) = diskbackend.manage_path(
+                    self.conn, path)
+
         self._change_backend(path, vol_object, parent_pool)
 
     def set_local_disk_to_clone(self, disk, sparse):
@@ -739,7 +707,12 @@ class DeviceDisk(Device):
         return self.device == self.DEVICE_DISK
 
     def can_be_empty(self):
-        return self.is_floppy() or self.is_cdrom()
+        if self.is_floppy() or self.is_cdrom():
+            return True
+        if self.type in ["file", "block", "dir", "volume", "network"]:
+            return False
+        # Don't error for unknown types
+        return True
 
     def _change_backend(self, path, vol_object, parent_pool):
         backend = diskbackend.StorageBackend(self.conn, path,
@@ -767,8 +740,7 @@ class DeviceDisk(Device):
         If true, this disk needs storage creation parameters or things
         will error.
         """
-        return (self._storage_backend and
-                not self._storage_backend.exists())
+        return not self._storage_backend.exists()
 
     def validate(self):
         if self.path is None:
@@ -781,21 +753,13 @@ class DeviceDisk(Device):
 
             return
 
-        if (self.type == DeviceDisk.TYPE_DIR and
-            not self.is_floppy()):
-            raise ValueError(_("The path '%s' must be a file or a "
-                               "device, not a directory") % self.path)
-
-        if not self._storage_backend:
-            return
-
-        if (not self._storage_backend.will_create_storage() and
-            not self._storage_backend.exists()):
+        if (not self._storage_backend.exists() and
+            not self._storage_backend.will_create_storage()):
             raise ValueError(
                 _("Must specify storage creation parameters for "
                   "non-existent path '%s'.") % self.path)
 
-        self._storage_backend.validate(self)
+        self._storage_backend.validate()
 
     def build_storage(self, meter):
         """
@@ -807,7 +771,8 @@ class DeviceDisk(Device):
         if not self._storage_backend.will_create_storage():
             return
 
-        meter = util.ensure_meter(meter)
+        meter = progress.ensure_meter(meter)
+        # pylint: disable=assignment-from-no-return
         vol_object = self._storage_backend.create(meter)
         self.storage_was_created = True
         if not vol_object:
@@ -828,25 +793,20 @@ class DeviceDisk(Device):
         """
         return self._storage_backend.is_size_conflict()
 
-    def is_conflict_disk(self, conn=None):
+    def is_conflict_disk(self):
         """
         check if specified storage is in use by any other VMs on passed
         connection.
 
         :returns: list of colliding VM names
         """
-        if not self.path:
-            return False
-        if not conn:
-            conn = self.conn
-
-        ret = self.path_in_use_by(conn, self.path,
+        ret = self.path_in_use_by(self.conn, self.path,
                                   shareable=self.shareable,
                                   read_only=self.read_only)
         return ret
 
 
-    def get_target_prefix(self, used_targets=None):
+    def get_target_prefix(self):
         """
         Returns the suggested disk target prefix (hd, xvd, sd ...) for the
         disk.
@@ -872,44 +832,27 @@ class DeviceDisk(Device):
             return _return("fd")
         elif self.bus == "ide":
             return _return("hd")
-        elif self.bus or not used_targets:
-            # sata, scsi, usb, sd
-            return _return("sd")
-
-        # If guest already has some disks defined
-        preforder = ["vd", "xvd", "sd", "hd"]
-        for pref in preforder:
-            for target in used_targets:
-                if target.startswith(pref):
-                    return _return(pref)
+        # sata, scsi, usb, sd
         return _return("sd")
 
-    def generate_target(self, skip_targets, pref_ctrl=None):
+
+    def generate_target(self, skip_targets):
         """
         Generate target device ('hda', 'sdb', etc..) for disk, excluding
-        any targets in 'skip_targets'.  If given the 'pref_ctrl'
-        parameter, it tries to select the target so that the disk is
-        mapped onto that controller.
+        any targets in 'skip_targets'.
         Sets self.target, and returns the generated value.
 
         :param skip_targets: list of targets to exclude
-        :param pref_ctrl: preferred controller to connect the disk to
         :returns: generated target
         """
-        prefix, maxnode = self.get_target_prefix(skip_targets)
+        prefix, maxnode = self.get_target_prefix()
         skip_targets = [t for t in skip_targets if t and t.startswith(prefix)]
         skip_targets.sort()
 
         def get_target():
             first_found = None
 
-            ran = range(maxnode)
-            if pref_ctrl is not None:
-                # We assume narrow SCSI bus and libvirt assigning 7
-                # (1-7, 8-14, etc.) devices per controller
-                ran = range(pref_ctrl * 7, (pref_ctrl + 1) * 7)
-
-            for i in ran:
+            for i in range(maxnode):
                 gen_t = prefix + self.num_to_target(i + 1)
                 if gen_t in skip_targets:
                     skip_targets.remove(gen_t)
@@ -926,14 +869,35 @@ class DeviceDisk(Device):
             self.target = ret
             return ret
 
-        if pref_ctrl is not None:
-            # This basically means that we either chose full
-            # controller or didn't add any
-            raise ValueError(_("Controller number %d for disk of type %s has "
-                               "no empty slot to use" % (pref_ctrl, prefix)))
-        else:
-            raise ValueError(_("Only %s disks for bus '%s' are supported"
-                               % (maxnode, self.bus)))
+        raise ValueError(
+            ngettext("Only %(number)s disk for bus '%(bus)s' are supported",
+                     "Only %(number)s disks for bus '%(bus)s' are supported",
+                     maxnode) %
+            {"number": maxnode, "bus": self.bus})
+
+    def change_bus(self, guest, newbus):
+        """
+        Change the bus value for an existing disk, which has some
+        follow on side effects.
+        """
+        if self.bus == newbus:
+            return
+
+        oldprefix = self.get_target_prefix()[0]
+        self.bus = newbus
+
+        self.address.clear()
+
+        if oldprefix == self.get_target_prefix()[0]:
+            return
+
+        used = [disk.target for disk in guest.devices.disk]
+
+        if self.target:
+            used.remove(self.target)
+
+        self.target = None
+        self.generate_target(used)
 
 
     ##################
@@ -949,27 +913,26 @@ class DeviceDisk(Device):
             # This likely isn't correct, but it's kind of a catch all
             # for virt types we don't know how to handle.
             return "ide"
-
-        if guest.os.is_arm_machvirt():
-            # We prefer virtio-scsi for machvirt, gets us hotplug
-            return "scsi"
         if self.is_disk() and guest.supports_virtiodisk():
             return "virtio"
-        if guest.os.is_pseries() and self.is_cdrom():
+        if (self.is_cdrom() and
+            guest.supports_virtioscsi() and
+            not guest.os.is_x86()):
+            # x86 long time default has been IDE CDROM, stick with that to
+            # avoid churn, but every newer virt arch that supports virtio-scsi
+            # should use it
             return "scsi"
         if guest.os.is_arm():
             return "sd"
         if guest.os.is_q35():
             return "sata"
-        if self.is_cdrom() and guest.os.is_s390x():
-            return "scsi"
         return "ide"
 
     def set_defaults(self, guest):
         if not self._device:
             self._device = self._get_device()
-        if not self._type:
-            self._type = self._get_default_type()
+        if not self._xmltype:
+            self._xmltype = self._get_default_type()
         if not self.driver_name:
             self.driver_name = self._get_default_driver_name()
         if not self.driver_type:
