@@ -11,11 +11,6 @@ from ..logger import log
 from ..xmlbuilder import XMLBuilder, XMLChildProperty, XMLProperty
 
 
-def _get_mode_prop(channel_type):
-    xpath = "./channel[@name='%s']/@mode" % channel_type
-    return XMLProperty(xpath)
-
-
 def _validate_port(name, val):
     if val is None:
         return val
@@ -44,56 +39,11 @@ class DeviceGraphics(Device):
     TYPE_RDP = "rdp"
     TYPE_SPICE = "spice"
 
-    CHANNEL_TYPE_MAIN     = "main"
-    CHANNEL_TYPE_DISPLAY  = "display"
-    CHANNEL_TYPE_INPUTS   = "inputs"
-    CHANNEL_TYPE_CURSOR   = "cursor"
-    CHANNEL_TYPE_PLAYBACK = "playback"
-    CHANNEL_TYPE_RECORD   = "record"
-
-    KEYMAP_LOCAL = "local"
-
-    @staticmethod
-    def valid_keymaps():
-        """
-        Return a list of valid keymap values.
-        """
-        from .. import hostkeymap
-
-        orig_list = list(hostkeymap.keytable.values())
-        sort_list = []
-
-        orig_list.sort()
-        for k in orig_list:
-            if k not in sort_list:
-                sort_list.append(k)
-
-        return sort_list
-
-    def __init__(self, *args, **kwargs):
-        Device.__init__(self, *args, **kwargs)
-
-        self._local_keymap = -1
-
-
-    _XML_PROP_ORDER = ["type", "gl", "_port", "_tlsPort", "autoport",
-                       "_keymap", "_listen",
+    _XML_PROP_ORDER = ["type", "gl", "_port", "_tlsPort", "autoport", "websocket",
+                       "keymap", "_listen",
                        "passwd", "display", "xauth"]
 
-    def _get_local_keymap(self):
-        if self._local_keymap == -1:
-            from .. import hostkeymap
-            self._local_keymap = hostkeymap.default_keymap()
-        return self._local_keymap
-
-    def _set_keymap(self, val):
-        if val == self.KEYMAP_LOCAL:
-            val = self._get_local_keymap()
-        self._keymap = val
-    def _get_keymap(self):
-        return self._keymap
-    _keymap = XMLProperty("./@keymap")
-    keymap = property(_get_keymap, _set_keymap)
+    keymap = XMLProperty("./@keymap")
 
     def _set_port(self, val):
         val = _validate_port("Port", val)
@@ -114,13 +64,7 @@ class DeviceGraphics(Device):
     tlsPort = property(_get_tlsport, _set_tlsport)
 
     autoport = XMLProperty("./@autoport", is_yesno=True)
-
-    channel_main_mode = _get_mode_prop(CHANNEL_TYPE_MAIN)
-    channel_display_mode = _get_mode_prop(CHANNEL_TYPE_DISPLAY)
-    channel_inputs_mode = _get_mode_prop(CHANNEL_TYPE_INPUTS)
-    channel_cursor_mode = _get_mode_prop(CHANNEL_TYPE_CURSOR)
-    channel_playback_mode = _get_mode_prop(CHANNEL_TYPE_PLAYBACK)
-    channel_record_mode = _get_mode_prop(CHANNEL_TYPE_RECORD)
+    websocket = XMLProperty("./@websocket", is_int=True)
 
     xauth = XMLProperty("./@xauth")
     display = XMLProperty("./@display")
@@ -188,7 +132,7 @@ class DeviceGraphics(Device):
     def _spice_supported(self):
         if not self.conn.is_qemu() and not self.conn.is_test():
             return False
-        # Spice has issues on some host arches, like ppc, so whitelist it
+        # Spice has issues on some host arches, like ppc, so allow it
         if self.conn.caps.host.cpu.arch not in ["i686", "x86_64"]:
             return False
         return True
@@ -235,12 +179,9 @@ class DeviceGraphics(Device):
         return None
 
     def _default_spice_gl(self, _guest):
-        if not self.conn.support.conn_spice_gl():  # pragma: no cover
-            raise ValueError(_("Host does not support spice GL"))
-
         # If spice GL but rendernode wasn't specified, hardcode
         # the first one
-        if not self.rendernode and self.conn.support.conn_spice_rendernode():
+        if not self.rendernode:
             for nodedev in self.conn.fetch_all_nodedevs():
                 if not nodedev.is_drm_render():
                     continue
