@@ -15,6 +15,7 @@ import libvirt
 
 from .logger import log
 from .storage import StoragePool, StorageVolume
+from . import xmlutil
 
 
 def _lookup_vol_by_path(conn, path):
@@ -34,7 +35,7 @@ def _lookup_vol_by_path(conn, path):
         # an empty error code
         if (e.get_error_code() and
             e.get_error_code() != libvirt.VIR_ERR_NO_STORAGE_VOL):
-            raise
+            raise  # pragma: no cover
         return None, e
 
 
@@ -49,29 +50,29 @@ def _lookup_vol_by_basename(pool, path):
         return pool.storageVolLookupByName(name)
 
 
-def _stat_disk(path):
-    """
-    Returns the tuple (isreg, size)
-    """
+def _get_block_size(path):  # pragma: no cover
+    try:
+        fd = os.open(path, os.O_RDONLY)
+        # os.SEEK_END is not present on all systems
+        size = os.lseek(fd, 0, 2)
+        os.close(fd)
+    except Exception:
+        size = 0
+    return size
+
+
+def _get_size(path):
     if not os.path.exists(path):
-        return True, 0
+        return 0
+    if _stat_is_block(path):
+        return _get_block_size(path)  # pragma: no cover
+    return os.path.getsize(path)
 
-    mode = os.stat(path)[stat.ST_MODE]
 
-    # os.path.getsize('/dev/..') can be zero on some platforms
-    if stat.S_ISBLK(mode):
-        try:
-            fd = os.open(path, os.O_RDONLY)
-            # os.SEEK_END is not present on all systems
-            size = os.lseek(fd, 0, 2)
-            os.close(fd)
-        except Exception:
-            size = 0
-        return False, size
-    elif stat.S_ISREG(mode):
-        return True, os.path.getsize(path)
-
-    return True, 0
+def _stat_is_block(path):
+    if not os.path.exists(path):
+        return False
+    return stat.S_ISBLK(os.stat(path)[stat.ST_MODE])
 
 
 def _check_if_path_managed(conn, path):
@@ -97,14 +98,14 @@ def _check_if_path_managed(conn, path):
         if verr:
             try:
                 vol = _lookup_vol_by_basename(pool, path)
-            except Exception:
+            except Exception:  # pragma: no cover
                 pass
-    except Exception as e:
+    except Exception as e:  # pragma: no cover
         vol = None
         pool = None
         verr = str(e)
 
-    if not vol and not pool and verr:
+    if not vol and not pool and verr:  # pragma: no cover
         raise ValueError(_("Cannot use storage %(path)s: %(err)s") %
             {'path': path, 'err': verr})
 
@@ -129,7 +130,7 @@ def manage_path(conn, path):
     If path is not managed, try to create a storage pool to probe the path
     """
     if not conn.support.conn_storage():
-        return None, None
+        return None, None  # pragma: no cover
     if not path:
         return None, None
 
@@ -160,20 +161,15 @@ def path_is_url(path):
     """
     Detect if path is a URL
     """
-    if not path:
-        return False
-    return bool(re.match(r"[a-zA-Z]+(\+[a-zA-Z]+)?://.*", path))
+    return bool(re.match(r"[a-zA-Z]+(\+[a-zA-Z]+)?://.*", path or ""))
 
 
 def path_is_network_vol(conn, path):
     """
     Detect if path is a network volume such as rbd, gluster, etc
     """
-    if not path:
-        return False
-
     for volxml in conn.fetch_all_vols():
-        if volxml.target_path == path:
+        if path and volxml.target_path == path:
             return volxml.type == "network"
     return False
 
@@ -182,25 +178,8 @@ def _get_dev_type(path, vol_xml, vol_object, pool_xml, remote):
     """
     Try to get device type for volume.
     """
-    if vol_xml:
-        if vol_xml.type:
-            return vol_xml.type
-
-        # If vol_xml.type is None the vol_xml.file_type can return only
-        # these types: block, network or file
-        if vol_xml.file_type == libvirt.VIR_STORAGE_VOL_BLOCK:
-            return "block"
-        elif vol_xml.file_type == libvirt.VIR_STORAGE_VOL_NETWORK:
-            return "network"
-
-    if vol_object:
-        t = vol_object.info()[0]
-        if t == StorageVolume.TYPE_FILE:
-            return "file"
-        elif t == StorageVolume.TYPE_BLOCK:
-            return "block"
-        elif t == StorageVolume.TYPE_NETWORK:
-            return "network"
+    if vol_xml and vol_xml.type:
+        return vol_xml.type
 
     if pool_xml:
         t = pool_xml.get_disk_type()
@@ -209,17 +188,33 @@ def _get_dev_type(path, vol_xml, vol_object, pool_xml, remote):
         elif t == StorageVolume.TYPE_NETWORK:
             return "network"
 
+    if vol_object:  # pragma: no cover
+        # This path is hard to test, because test suite XML always has
+        # the vol_xml.type set
+        t = vol_object.info()[0]
+        if t == StorageVolume.TYPE_FILE:
+            return "file"
+        elif t == StorageVolume.TYPE_BLOCK:
+            return "block"
+        elif t == StorageVolume.TYPE_NETWORK:
+            return "network"
+
     if path:
         if path_is_url(path):
             return "network"
 
-        if not remote:
+        if remote:
+            if not _can_auto_manage(path):
+                # Just a heurisitic, if this path is one of the ones
+                # we don't try to auto-import, then consider it a
+                # block device, because managing those correctly is difficult
+                return "block"
+
+        else:
             if os.path.isdir(path):
                 return "dir"
-            elif _stat_disk(path)[0]:
-                return "file"
-            else:
-                return "block"
+            elif _stat_is_block(path):
+                return "block"  # pragma: no cover
 
     return "file"
 
@@ -240,7 +235,7 @@ def path_definitely_exists(conn, path):
 
         if not conn.is_remote():
             return os.path.exists(path)
-    except Exception:
+    except Exception:  # pragma: no cover
         pass
 
     return False
@@ -250,8 +245,11 @@ def path_definitely_exists(conn, path):
 # ACL/path perm helpers #
 #########################
 
+SETFACL = "setfacl"
+
+
 def _fix_perms_acl(dirname, username):
-    cmd = ["setfacl", "--modify", "user:%s:x" % username, dirname]
+    cmd = [SETFACL, "--modify", "user:%s:x" % username, dirname]
     proc = subprocess.Popen(cmd,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
@@ -273,8 +271,8 @@ def _fix_perms_chmod(dirname):
     if os.stat(dirname).st_mode != newmode:
         # Trying to change perms on vfat at least doesn't work
         # but also doesn't seem to error. Try and detect that
-        raise ValueError(_("Permissions on '%s' did not stick") %
-                         dirname)
+        raise ValueError(  # pragma: no cover
+                _("Permissions on '%s' did not stick") % dirname)
 
 
 def set_dirs_searchable(dirlist, username):
@@ -293,7 +291,7 @@ def set_dirs_searchable(dirlist, username):
         try:
             # If we reach here, ACL setting failed, try chmod
             _fix_perms_chmod(dirname)
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             errdict[dirname] = str(e)
 
     return errdict
@@ -303,18 +301,15 @@ def _is_dir_searchable(dirname, uid, username):
     """
     Check if passed directory is searchable by uid
     """
-    if "VIRTINST_TEST_SUITE" in os.environ:
-        return True
-
     try:
         statinfo = os.stat(dirname)
-    except OSError:
+    except OSError:  # pragma: no cover
         return False
 
     if uid == statinfo.st_uid:
         flag = stat.S_IXUSR
     elif uid == statinfo.st_gid:
-        flag = stat.S_IXGRP
+        flag = stat.S_IXGRP  # pragma: no cover
     else:
         flag = stat.S_IXOTH
 
@@ -328,11 +323,11 @@ def _is_dir_searchable(dirname, uid, username):
                                 stdout=subprocess.PIPE,
                                 stderr=subprocess.PIPE)
         out, err = proc.communicate()
-    except OSError:
+    except OSError:  # pragma: no cover
         log.debug("Didn't find the getfacl command.")
         return False
 
-    if proc.returncode != 0:
+    if proc.returncode != 0:  # pragma: no cover
         log.debug("Cmd '%s' failed: %s", cmd, err)
         return False
 
@@ -390,19 +385,24 @@ class _StorageBase(object):
             self._parent_pool_xml = StoragePool(self._conn,
                 parsexml=self.get_parent_pool().XMLDesc(0))
         return self._parent_pool_xml
-    def validate(self, disk):
+    def validate(self):
         raise NotImplementedError()
     def get_path(self):
         raise NotImplementedError()
+    def is_stub(self):
+        return False
 
     # Storage creation routines
     def is_size_conflict(self):
         raise NotImplementedError()
-    def create(self, progresscb):
-        raise NotImplementedError()
     def will_create_storage(self):
         raise NotImplementedError()
 
+    def create(self, progresscb):
+        ignore = progresscb  # pragma: no cover
+        raise xmlutil.DevError(
+            "%s can't create storage" % self.__class__.__name__)
+
 
 class _StorageCreator(_StorageBase):
     """
@@ -423,7 +423,11 @@ class _StorageCreator(_StorageBase):
     ##############
 
     def create(self, progresscb):
-        raise NotImplementedError()
+        raise NotImplementedError
+    def validate(self):
+        raise NotImplementedError
+    def get_size(self):
+        raise NotImplementedError
 
     def get_path(self):
         if self._vol_install and not self._path:
@@ -432,11 +436,8 @@ class _StorageCreator(_StorageBase):
             if self.get_dev_type() == "network":
                 self._path = self._vol_install.name
             else:
-                sep = "/"
-                if xmlobj.target_path == "" or xmlobj.target_path[-1] == '/':
-                    sep = ""
-                self._path = (xmlobj.target_path + sep +
-                              self._vol_install.name)
+                self._path = os.path.join(
+                        xmlobj.target_path, self._vol_install.name)
         return self._path
 
     def get_vol_install(self):
@@ -444,12 +445,6 @@ class _StorageCreator(_StorageBase):
     def get_vol_xml(self):
         return self._vol_install
 
-    def get_size(self):
-        if self._size is None:
-            self._size = (float(self._vol_install.capacity) /
-                          1024.0 / 1024.0 / 1024.0)
-        return self._size
-
     def get_dev_type(self):
         if not self._dev_type:
             self._dev_type = _get_dev_type(self._path, self._vol_install, None,
@@ -463,25 +458,6 @@ class _StorageCreator(_StorageBase):
                 return self._vol_install.format
         return "raw"
 
-    def validate(self, disk):
-        if disk.device in ["floppy", "cdrom"]:
-            raise ValueError(_("Cannot create storage for %s device.") %
-                             disk.device)
-
-        if self._vol_install:
-            self._vol_install.validate()
-            return
-
-        if self._size is None:
-            raise ValueError(_("size is required for non-existent disk "
-                               "'%s'" % self.get_path()))
-
-        err, msg = self.is_size_conflict()
-        if err:
-            raise ValueError(msg)
-        if msg:
-            log.warning(msg)
-
     def will_create_storage(self):
         return True
     def get_vol_object(self):
@@ -494,6 +470,28 @@ class _StorageCreator(_StorageBase):
         return False
 
 
+class ManagedStorageCreator(_StorageCreator):
+    """
+    Handles storage creation via libvirt APIs. All the actual creation
+    logic lives in StorageVolume, this is mostly about pulling out bits
+    from that class and mapping them to DeviceDisk elements
+    """
+    def __init__(self, conn, vol_install):
+        _StorageCreator.__init__(self, conn)
+
+        self._pool = vol_install.pool
+        self._vol_install = vol_install
+
+    def create(self, progresscb):
+        return self._vol_install.install(meter=progresscb)
+    def is_size_conflict(self):
+        return self._vol_install.is_size_conflict()
+    def validate(self):
+        return self._vol_install.validate()
+    def get_size(self):
+        return float(self._vol_install.capacity) / 1024.0 / 1024.0 / 1024.0
+
+
 class CloneStorageCreator(_StorageCreator):
     """
     Handles manually copying local files for Cloner
@@ -510,16 +508,19 @@ class CloneStorageCreator(_StorageCreator):
         self._size = size
         self._sparse = sparse
 
+    def get_size(self):
+        return self._size
+
     def is_size_conflict(self):
         ret = False
         msg = None
         if self.get_dev_type() == "block":
-            avail = _stat_disk(self._path)[1]
+            avail = _get_size(self._path)  # pragma: no cover
         else:
-            vfs = os.statvfs(os.path.dirname(self._path))
+            vfs = os.statvfs(os.path.dirname(os.path.abspath(self._path)))
             avail = vfs.f_frsize * vfs.f_bavail
         need = int(self._size) * 1024 * 1024 * 1024
-        if need > avail:
+        if need > avail:  # pragma: no cover
             if self._sparse:
                 msg = _("The filesystem will not have enough free space"
                         " to fully allocate the sparse file when the guest"
@@ -530,15 +531,28 @@ class CloneStorageCreator(_StorageCreator):
 
 
             if msg:
-                msg += (_(" %d M requested > %d M available") %
-                        ((need // (1024 * 1024)), (avail // (1024 * 1024))))
+                msg += " "
+                msg += (_("%(mem1)s M requested > %(mem2)s M available") %
+                        {"mem1": (need // (1024 * 1024)),
+                         "mem2": (avail // (1024 * 1024))})
         return (ret, msg)
 
+    def validate(self):
+        if self._size is None:  # pragma: no cover
+            raise ValueError(_("size is required for non-existent disk "
+                               "'%s'" % self.get_path()))
+
+        err, msg = self.is_size_conflict()
+        if err:
+            raise ValueError(msg)  # pragma: no cover
+        if msg:
+            log.warning(msg)  # pragma: no cover
+
     def create(self, progresscb):
         text = (_("Cloning %(srcfile)s") %
                 {'srcfile': os.path.basename(self._input_path)})
 
-        size_bytes = int(self.get_size()) * 1024 * 1024 * 1024
+        size_bytes = int(self.get_size() * 1024 * 1024 * 1024)
         progresscb.start(filename=self._output_path, size=size_bytes,
                          text=text)
 
@@ -546,7 +560,7 @@ class CloneStorageCreator(_StorageCreator):
         self._clone_local(progresscb, size_bytes)
 
     def _clone_local(self, meter, size_bytes):
-        if self._input_path == "/dev/null":
+        if self._input_path == "/dev/null":  # pragma: no cover
             # Not really sure why this check is here,
             # but keeping for compat
             log.debug("Source dev was /dev/null. Skipping")
@@ -577,7 +591,7 @@ class CloneStorageCreator(_StorageCreator):
                       self._input_path, self._output_path,
                       sparse, clone_block_size)
 
-        zeros = '\0' * 4096
+        zeros = b'\0' * 4096
 
         src_fd, dst_fd = None, None
         try:
@@ -598,15 +612,20 @@ class CloneStorageCreator(_StorageCreator):
                         os.lseek(dst_fd, s, 1)
                     else:
                         b = os.write(dst_fd, l)
-                        if s != b:
+                        if s != b:  # pragma: no cover
                             meter.end(i)
                             break
                     i += s
                     if i < size_bytes:
                         meter.update(i)
-            except OSError as e:
-                raise RuntimeError(_("Error cloning diskimage %s to %s: %s") %
-                                (self._input_path, self._output_path, str(e)))
+            except OSError as e:  # pragma: no cover
+                log.debug("Error while cloning", exc_info=True)
+                msg = (_("Error cloning diskimage "
+                         "%(inputpath)s to %(outputpath)s: %(error)s") %
+                         {"inputpath": self._input_path,
+                          "outputpath": self._output_path,
+                          "error": str(e)})
+                raise RuntimeError(msg) from None
         finally:
             if src_fd is not None:
                 os.close(src_fd)
@@ -614,22 +633,45 @@ class CloneStorageCreator(_StorageCreator):
                 os.close(dst_fd)
 
 
-class ManagedStorageCreator(_StorageCreator):
+class StorageBackendStub(_StorageBase):
     """
-    Handles storage creation via libvirt APIs. All the actual creation
-    logic lives in StorageVolume, this is mostly about pulling out bits
-    from that class and mapping them to DeviceDisk elements
+    Class representing a storage path for a parsed XML disk, that we
+    don't want to do slow resolving of unless requested
     """
-    def __init__(self, conn, vol_install):
-        _StorageCreator.__init__(self, conn)
+    def __init__(self, conn, path, dev_type, driver_type):
+        _StorageBase.__init__(self, conn)
+        self._path = path
+        self._dev_type = dev_type
+        self._driver_type = driver_type
 
-        self._pool = vol_install.pool
-        self._vol_install = vol_install
 
-    def create(self, progresscb):
-        return self._vol_install.install(meter=progresscb)
+    def get_path(self):
+        return self._path
+    def get_vol_object(self):
+        return None
+    def get_vol_xml(self):
+        return None
+    def get_parent_pool(self):
+        return None
+    def get_size(self):
+        return 0
+    def exists(self):
+        return True
+    def get_dev_type(self):
+        return self._dev_type
+    def get_driver_type(self):
+        return self._driver_type
+
+    def validate(self):
+        return
+    def get_vol_install(self):
+        return None
     def is_size_conflict(self):
-        return self._vol_install.is_size_conflict()
+        return (False, None)
+    def is_stub(self):
+        return True
+    def will_create_storage(self):
+        return False
 
 
 class StorageBackend(_StorageBase):
@@ -648,8 +690,8 @@ class StorageBackend(_StorageBase):
             self._path = None
 
         if self._vol_object and not self._parent_pool:
-            raise RuntimeError(
-                "programming error: parent_pool must be specified")
+            raise xmlutil.DevError(
+                "parent_pool must be specified")
 
         # Cached bits
         self._vol_xml = None
@@ -689,15 +731,15 @@ class StorageBackend(_StorageBase):
             if self._vol_object:
                 ret = self.get_vol_xml().capacity
             elif self._path:
-                ret = _stat_disk(self._path)[1]
+                ret = _get_size(self._path)
             self._size = (float(ret) / 1024.0 / 1024.0 / 1024.0)
         return self._size
 
     def exists(self):
         if self._exists is None:
-            if self._path is None:
+            if self._vol_object:
                 self._exists = True
-            elif self._vol_object:
+            elif self._path is None:
                 self._exists = True
             elif (not self.get_dev_type() == "network" and
                   not self._conn.is_remote() and
@@ -738,8 +780,7 @@ class StorageBackend(_StorageBase):
                 return ret
         return None
 
-    def validate(self, disk):
-        ignore = disk
+    def validate(self):
         return
     def get_vol_install(self):
         return None
@@ -747,7 +788,3 @@ class StorageBackend(_StorageBase):
         return (False, None)
     def will_create_storage(self):
         return False
-    def create(self, progresscb):
-        ignore = progresscb
-        raise RuntimeError("programming error: %s can't create storage" %
-            self.__class__.__name__)
