@@ -45,8 +45,12 @@ class _CapsBlock(_HasValues):
         return [e.name for e in self.enums]
 
     def get_enum(self, name):
-        d = dict((e.name, e) for e in self.enums)
-        return d[name]
+        for enum in self.enums:
+            if enum.name == name:
+                return enum
+        # Didn't find a match. Could be talking to older libvirt, or
+        # driver with incomplete info. Return a stub enum
+        return _Enum(self.conn)
 
 
 def _make_capsblock(xml_root_name):
@@ -95,6 +99,7 @@ class _Devices(_CapsBlock):
     XML_NAME = "devices"
     hostdev = XMLChildProperty(_make_capsblock("hostdev"), is_single=True)
     disk = XMLChildProperty(_make_capsblock("disk"), is_single=True)
+    video = XMLChildProperty(_make_capsblock("video"), is_single=True)
 
 
 class _Features(_CapsBlock):
@@ -157,7 +162,7 @@ class DomainCapabilities(XMLBuilder):
             try:
                 xml = conn.getDomainCapabilities(emulator, arch,
                     machine, hvtype)
-            except Exception:
+            except Exception:  # pragma: no cover
                 log.debug("Error fetching domcapabilities XML",
                     exc_info=True)
 
@@ -176,9 +181,11 @@ class DomainCapabilities(XMLBuilder):
     # validate anything the user explicitly enters.
     _uefi_arch_patterns = {
         "i686": [
+            r".*edk2-i386-.*\.fd",  # upstream qemu
             r".*ovmf-ia32.*",  # fedora, gerd's firmware repo
         ],
         "x86_64": [
+            r".*edk2-x86_64-.*\.fd",  # upstream qemu
             r".*OVMF_CODE\.fd",  # RHEL
             r".*ovmf-x64/OVMF.*\.fd",  # gerd's firmware repo
             r".*ovmf-x86_64-.*",  # SUSE
@@ -188,9 +195,11 @@ class DomainCapabilities(XMLBuilder):
             r".*AAVMF_CODE\.fd",  # RHEL
             r".*aarch64/QEMU_EFI.*",  # gerd's firmware repo
             r".*aarch64.*",  # generic attempt at a catchall
+            r".*edk2-aarch64-code\.fd",  # upstream qemu
         ],
         "armv7l": [
             r".*arm/QEMU_EFI.*",  # fedora, gerd's firmware repo
+            r".*edk2-arm-code\.fd"  # upstream qemu
         ],
     }
 
@@ -199,7 +208,7 @@ class DomainCapabilities(XMLBuilder):
         Search the loader paths for one that matches the passed arch
         """
         if not self.arch_can_uefi():
-            return
+            return  # pragma: no cover
 
         patterns = self._uefi_arch_patterns.get(self.arch)
         for pattern in patterns:
@@ -241,7 +250,7 @@ class DomainCapabilities(XMLBuilder):
     def supports_safe_host_model(self):
         """
         Return True if domcaps reports support for cpu mode=host-model.
-        host-model infact predates this support, however it wasn't
+        host-model in fact predates this support, however it wasn't
         general purpose safe prior to domcaps advertisement.
         """
         for m in self.cpu.modes:
@@ -264,30 +273,27 @@ class DomainCapabilities(XMLBuilder):
     def _convert_mode_to_cpu(self, xml):
         root = ET.fromstring(xml)
         root.tag = "cpu"
-        root.attrib = None
+        root.attrib = {}
         arch = ET.SubElement(root, "arch")
         arch.text = self.arch
         return ET.tostring(root, encoding="unicode")
 
     def _get_expanded_cpu(self, mode):
         cpuXML = self._convert_mode_to_cpu(mode.get_xml())
-        log.debug("CPU XML for security flag baseline: %s", cpuXML)
+        log.debug("Generated CPU XML for security flag baseline:\n%s", cpuXML)
 
         try:
             expandedXML = self.conn.baselineHypervisorCPU(
                     self.path, self.arch, self.machine, self.domain, [cpuXML],
                     libvirt.VIR_CONNECT_BASELINE_CPU_EXPAND_FEATURES)
-        except libvirt.libvirtError:
+        except (libvirt.libvirtError, AttributeError):
             expandedXML = self.conn.baselineCPU([cpuXML],
                     libvirt.VIR_CONNECT_BASELINE_CPU_EXPAND_FEATURES)
 
-        log.debug("Expanded CPU XML: %s", expandedXML)
-
         return DomainCpu(self.conn, expandedXML)
 
-    _features = None
-
-    def get_cpu_security_features(self):
+    def _lookup_cpu_security_features(self):
+        ret = []
         sec_features = [
                 'spec-ctrl',
                 'ssbd',
@@ -295,27 +301,30 @@ class DomainCapabilities(XMLBuilder):
                 'virt-ssbd',
                 'md-clear']
 
-        if self._features:
-            return self._features
-
-        self._features = []
-
         for m in self.cpu.modes:
             if m.name != "host-model" or not m.supported:
-                continue
+                continue  # pragma: no cover
 
             try:
                 cpu = self._get_expanded_cpu(m)
-            except libvirt.libvirtError as e:
+            except libvirt.libvirtError as e:  # pragma: no cover
                 log.warning(_("Failed to get expanded CPU XML: %s"), e)
                 break
 
             for feature in cpu.features:
                 if feature.name in sec_features:
-                    self._features.append(feature.name)
+                    ret.append(feature.name)
 
+        log.debug("Found host-model security features: %s", ret)
+        return ret
+
+    _features = None
+    def get_cpu_security_features(self):
+        if self._features is None:
+            self._features = self._lookup_cpu_security_features() or []
         return self._features
 
+
     def supports_sev_launch_security(self):
         """
         Returns False if either libvirt doesn't advertise support for SEV at
@@ -324,6 +333,14 @@ class DomainCapabilities(XMLBuilder):
         """
         return bool(self.features.sev.supported)
 
+    def supports_video_bochs(self):
+        """
+        Returns False if either libvirt or qemu do not have support to bochs
+        video type.
+        """
+        models = self.devices.video.get_enum("modelType").get_values()
+        return bool("bochs" in models)
+
     XML_NAME = "domainCapabilities"
     os = XMLChildProperty(_OS, is_single=True)
     cpu = XMLChildProperty(_CPU, is_single=True)
