@@ -12,11 +12,19 @@ import re
 
 from gi.repository import Libosinfo
 
+from . import xmlutil
 from .logger import log
 
 
-def _in_testsuite():
-    return "VIRTINST_TEST_SUITE" in os.environ
+def _media_create_from_location(location):
+    if not hasattr(Libosinfo.Media, "create_from_location_with_flags"):
+        return Libosinfo.Media.create_from_location(  # pragma: no cover
+                location, None)
+
+    # We prefer this API, because by default it will not
+    # reject non-bootable media, like debian s390x
+    # pylint: disable=no-member
+    return Libosinfo.Media.create_from_location_with_flags(location, None, 0)
 
 
 ###################
@@ -201,7 +209,8 @@ class _OSDB(object):
             oslist = db.get_os_list()
             for o in _OsinfoIter(oslist):
                 osi = _OsVariant(o)
-                allvariants[osi.name] = osi
+                for name in osi.get_short_ids():
+                    allvariants[name] = osi
 
             self.__all_variants = allvariants
         return self.__all_variants
@@ -219,12 +228,13 @@ class _OSDB(object):
             raise ValueError(_("Unknown libosinfo ID '%s'") % full_id)
 
     def lookup_os(self, key, raise_error=False):
-        if key in self._aliases:
+        if key not in self._all_variants and key in self._aliases:
             alias = self._aliases[key]
             # Added 2018-10-02. Maybe remove aliases in a year
-            log.warning(
-                _("OS name '%s' is deprecated, using '%s' instead. "
-                  "This alias will be removed in the future."), key, alias)
+            msg = (_("OS name '%(oldname)s' is deprecated, using '%(newname)s' "
+                    "instead. This alias will be removed in the future.") %
+                    {"oldname": key, "newname": alias})
+            log.warning(msg)
             key = alias
 
         ret = self._all_variants.get(key)
@@ -235,7 +245,7 @@ class _OSDB(object):
 
     def guess_os_by_iso(self, location):
         try:
-            media = Libosinfo.Media.create_from_location(location, None)
+            media = _media_create_from_location(location)
         except Exception as e:
             log.debug("Error creating libosinfo media object: %s", str(e))
             return None
@@ -248,7 +258,7 @@ class _OSDB(object):
         if location.startswith("/"):
             location = "file://" + location
 
-        if _in_testsuite() and not location.startswith("file:"):
+        if xmlutil.in_testsuite() and not location.startswith("file:"):
             # We have mock network tests, but we don't want to pass the
             # fake URL to libosinfo because it slows down the testcase
             return None
@@ -260,10 +270,17 @@ class _OSDB(object):
                 "location=%s : %s", location, str(e))
             return None
 
-        osobj, treeobj = self._os_loader.get_db().guess_os_from_tree(tree)
-        if not osobj:
-            return None  # pragma: no cover
-        return osobj.get_short_id(), treeobj
+        db = self._os_loader.get_db()
+        if hasattr(db, "identify_tree"):
+            # osinfo_db_identify_tree is part of libosinfo 1.6.0
+            if not db.identify_tree(tree):
+                return None  # pragma: no cover
+            return tree.get_os().get_short_id(), _OsTree(tree)
+        else:  # pragma: no cover
+            osobj, treeobj = self._os_loader.get_db().guess_os_from_tree(tree)
+            if not osobj:
+                return None  # pragma: no cover
+            return osobj.get_short_id(), _OsTree(treeobj)
 
     def list_os(self):
         """
@@ -271,8 +288,8 @@ class _OSDB(object):
         """
         sortmap = {}
 
-        for name, osobj in self._all_variants.items():
-            sortmap[name] = osobj
+        for osobj in self._all_variants.values():
+            sortmap[osobj.name] = osobj
 
         return _sort(sortmap)
 
@@ -305,8 +322,9 @@ class _OsResources:
 
     def _get_key(self, resources, key, arch):
         for checkarch in [arch, "all"]:
-            if checkarch in resources and key in resources[checkarch]:
-                return resources[checkarch][key]
+            val = resources.get(checkarch, {}).get(key, -1)
+            if val != -1:
+                return val
 
     def _get_minimum_key(self, key, arch):
         val = self._get_key(self._minimum, key, arch)
@@ -349,9 +367,15 @@ class _OsVariant(object):
         self._os = o
         self._family = self._os and self._os.get_family() or None
 
+        self._short_ids = ["generic"]
+        if self._os:
+            self._short_ids = [self._os.get_short_id()]
+            if hasattr(self._os, "get_short_id_list"):
+                self._short_ids = self._os.get_short_id_list()
+        self.name = self._short_ids[0]
+
         self.full_id = self._os and self._os.get_id() or None
-        self.name = self._os and self._os.get_short_id() or "generic"
-        self.label = self._os and self._os.get_name() or "Generic default"
+        self.label = self._os and self._os.get_name() or _("Generic OS")
         self.codename = self._os and self._os.get_codename() or ""
         self.distro = self._os and self._os.get_distro() or ""
         self.version = self._os and self._os.get_version() or None
@@ -406,7 +430,7 @@ class _OsVariant(object):
             return []
         return list(_OsinfoIter(self._os.get_all_devices()))
 
-    def _device_filter(self, devids=None, cls=None):
+    def _device_filter(self, devids=None, cls=None, extra_devs=None):
         ret = []
         devids = devids or []
         for dev in self._get_all_devices():
@@ -415,6 +439,13 @@ class _OsVariant(object):
             if cls and not re.match(cls, dev.get_class()):
                 continue
             ret.append(dev.get_name())
+
+        extra_devs = extra_devs or []
+        for dev in extra_devs:
+            if dev.get_id() not in devids:
+                continue
+            ret.append(dev.get_name())
+
         return ret
 
 
@@ -443,9 +474,9 @@ class _OsVariant(object):
         if release_status == "rolling":
             return False
 
-        # If no EOL is present, assume EOL if release was > 5 years ago
+        # If no EOL is present, assume EOL if release was > 10 years ago
         if rel is not None:
-            rel5 = _glib_to_datetime(rel) + datetime.timedelta(days=365 * 5)
+            rel5 = _glib_to_datetime(rel) + datetime.timedelta(days=365 * 10)
             return now > rel5
         return False
 
@@ -463,6 +494,9 @@ class _OsVariant(object):
     def is_windows(self):
         return self._family in ['win9x', 'winnt', 'win16']
 
+    def get_short_ids(self):
+        return self._short_ids[:]
+
     def broken_uefi_with_hyperv(self):
         # Some windows versions are broken with hyperv enlightenments + UEFI
         # https://bugzilla.redhat.com/show_bug.cgi?id=1185253
@@ -477,68 +511,75 @@ class _OsVariant(object):
     def supported_netmodels(self):
         return self._device_filter(cls="net")
 
-    def supports_usbtablet(self):
+    def supports_usbtablet(self, extra_devs=None):
         # If no OS specified, still default to tablet
         if not self._os:
             return True
 
         devids = ["http://usb.org/usb/80ee/0021"]
-        return bool(self._device_filter(devids=devids))
+        return bool(self._device_filter(devids=devids, extra_devs=extra_devs))
 
-    def supports_virtiodisk(self):
+    def supports_virtiodisk(self, extra_devs=None):
         # virtio-block and virtio1.0-block
         devids = ["http://pcisig.com/pci/1af4/1001",
                   "http://pcisig.com/pci/1af4/1042"]
-        return bool(self._device_filter(devids=devids))
+        return bool(self._device_filter(devids=devids, extra_devs=extra_devs))
 
-    def supports_virtioscsi(self):
+    def supports_virtioscsi(self, extra_devs=None):
         # virtio-scsi and virtio1.0-scsi
         devids = ["http://pcisig.com/pci/1af4/1004",
                   "http://pcisig.com/pci/1af4/1048"]
-        return bool(self._device_filter(devids=devids))
+        return bool(self._device_filter(devids=devids, extra_devs=extra_devs))
 
-    def supports_virtionet(self):
+    def supports_virtionet(self, extra_devs=None):
         # virtio-net and virtio1.0-net
         devids = ["http://pcisig.com/pci/1af4/1000",
                   "http://pcisig.com/pci/1af4/1041"]
-        return bool(self._device_filter(devids=devids))
+        return bool(self._device_filter(devids=devids, extra_devs=extra_devs))
 
-    def supports_virtiorng(self):
+    def supports_virtiorng(self, extra_devs=None):
         # virtio-rng and virtio1.0-rng
         devids = ["http://pcisig.com/pci/1af4/1005",
                   "http://pcisig.com/pci/1af4/1044"]
-        return bool(self._device_filter(devids=devids))
+        return bool(self._device_filter(devids=devids, extra_devs=extra_devs))
 
-    def supports_virtioserial(self):
+    def supports_virtioballoon(self, extra_devs=None):
+        # virtio-balloon and virtio1.0-balloon
+        devids = ["http://pcisig.com/pci/1af4/1002",
+                  "http://pcisig.com/pci/1af4/1045"]
+        return bool(self._device_filter(devids=devids, extra_devs=extra_devs))
+
+    def supports_virtioserial(self, extra_devs=None):
         devids = ["http://pcisig.com/pci/1af4/1003",
                   "http://pcisig.com/pci/1af4/1043"]
-        if self._device_filter(devids=devids):
+        if self._device_filter(devids=devids, extra_devs=extra_devs):
             return True
         # osinfo data was wrong for RHEL/centos here until Oct 2018
         # Remove this hack after 6 months or so
         return self._is_related_to("rhel6.0")
 
-    def supports_virtioinput(self):
+    def supports_virtioinput(self, extra_devs=None):
         # virtio1.0-input
         devids = ["http://pcisig.com/pci/1af4/1052"]
-        return bool(self._device_filter(devids=devids))
+        return bool(self._device_filter(devids=devids, extra_devs=extra_devs))
 
-    def supports_usb3(self):
+    def supports_usb3(self, extra_devs=None):
         # qemu-xhci
         devids = ["http://pcisig.com/pci/1b36/0004"]
-        return bool(self._device_filter(devids=devids))
+        return bool(self._device_filter(devids=devids, extra_devs=extra_devs))
 
-    def supports_virtio1(self):
+    def supports_virtio1(self, extra_devs=None):
         # Use virtio1.0-net device as a proxy for virtio1.0 as a whole
         devids = ["http://pcisig.com/pci/1af4/1041"]
-        return bool(self._device_filter(devids=devids))
+        return bool(self._device_filter(devids=devids, extra_devs=extra_devs))
 
-    def supports_chipset_q35(self):
+    def supports_chipset_q35(self, extra_devs=None):
         # For our purposes, check for the union of q35 + virtio1.0 support
-        if self.supports_virtionet() and not self.supports_virtio1():
+        if (self.supports_virtionet(extra_devs=extra_devs) and
+            not self.supports_virtio1(extra_devs=extra_devs)):
             return False
         devids = ["http://qemu.org/chipset/x86/q35"]
-        return bool(self._device_filter(devids=devids))
+        return bool(self._device_filter(devids=devids, extra_devs=extra_devs))
 
     def get_recommended_resources(self):
         minimum = self._os and self._os.get_minimum_resources() or None
@@ -586,8 +627,36 @@ class _OsVariant(object):
 
         return "inst.repo"
 
+    def _get_generic_location(self, treelist, arch, profile):
+        if not hasattr(Libosinfo.Tree, "get_os_variants"):  # pragma: no cover
+            for tree in treelist:
+                if tree.get_architecture() == arch:
+                    return tree.get_url()
+            return None
 
-    def get_location(self, arch):
+        fallback_tree = None
+        if profile == "jeos":
+            profile = "Server"
+        elif profile == "desktop":
+            profile = "Workstation"
+        elif not profile:
+            profile = "Everything"
+
+        for tree in treelist:
+            if tree.get_architecture() != arch:
+                continue
+
+            variant_list = tree.get_os_variants()
+            fallback_tree = tree
+            for variant in _OsinfoIter(variant_list):
+                if profile in variant.get_name():
+                    return tree.get_url()
+
+        if fallback_tree:
+            return fallback_tree.get_url()
+        return None
+
+    def get_location(self, arch, profile=None):
         treelist = []
         if self._os:
             treelist = list(_OsinfoIter(self._os.get_tree_list()))
@@ -599,20 +668,69 @@ class _OsVariant(object):
         # Some distros have more than one URL for a specific architecture,
         # which is the case for Fedora and different variants (Server,
         # Workstation). Later on, we'll have to differentiate that and return
-        # the right one.
-        for tree in treelist:
-            if tree.get_architecture() == arch:
-                return tree.get_url()
+        # the right one. However, for now, let's just rely on returning the
+        # most generic tree possible.
+        location = self._get_generic_location(treelist, arch, profile)
+        if location:
+            return location
 
         raise RuntimeError(
-            _("OS '%s' does not have a URL location for the %s architecture") %
-            (self.name, arch))
+            _("OS '%(osname)s' does not have a URL location "
+              "for the architecture '%(archname)s'") %
+            {"osname": self.name, "archname": arch})
 
     def get_install_script_list(self):
         if not self._os:
             return []  # pragma: no cover
         return list(_OsinfoIter(self._os.get_install_script_list()))
 
+    def _get_installable_drivers(self, arch):
+        if not self._os:
+            return []
+
+        installable_drivers = []
+        device_drivers = list(_OsinfoIter(self._os.get_device_drivers()))
+        for device_driver in device_drivers:
+            if arch != "all" and device_driver.get_architecture() != arch:
+                continue
+
+            installable_drivers.append(device_driver)
+        return installable_drivers
+
+    def _get_pre_installable_drivers(self, arch):
+        installable_drivers = self._get_installable_drivers(arch)
+        pre_inst_drivers = []
+        for driver in installable_drivers:
+            if driver.get_pre_installable():
+                pre_inst_drivers.append(driver)
+        return pre_inst_drivers
+
+    def _get_drivers_location(self, drivers):
+        locations = []
+        for driver in drivers:
+            filenames = driver.get_files()
+            for filename in filenames:
+                location = os.path.join(driver.get_location(), filename)
+                locations.append(location)
+        return locations
+
+    def get_pre_installable_drivers_location(self, arch):
+        pre_inst_drivers = self._get_pre_installable_drivers(arch)
+
+        return self._get_drivers_location(pre_inst_drivers)
+
+    def get_pre_installable_devices(self, arch):
+        drivers = self._get_pre_installable_drivers(arch)
+        devices = []
+        for driver in drivers:
+            devices += list(_OsinfoIter(driver.get_devices()))
+        return devices
+
+    def supports_unattended_drivers(self, arch):
+        if self._get_pre_installable_drivers(arch):
+            return True
+        return False
+
 
 class _OsMedia(object):
     def __init__(self, osinfo_media):
@@ -630,7 +748,18 @@ class _OsMedia(object):
         for variant in variants:
             if "netinst" in variant.get_id():
                 return True
-        return False
+        return False  # pragma: no cover
 
     def get_install_script_list(self):
         return list(_OsinfoIter(self._media.get_install_script_list()))
+
+    def get_osinfo_media(self):
+        return self._media
+
+
+class _OsTree(object):
+    def __init__(self, osinfo_tree):
+        self._tree = osinfo_tree
+
+    def get_osinfo_tree(self):
+        return self._tree
