@@ -8,6 +8,8 @@
 
 import libvirt
 
+from . import xmlutil
+
 
 def _check_function(function, flag, run_args, data):
     """
@@ -63,9 +65,8 @@ def _version_str_to_int(verstr):
         return 0
 
     if verstr.count(".") != 2:
-        raise RuntimeError(  # pragma: no cover
-            "programming error: version string '%s' needs "
-            "two '.' in it.")
+        raise xmlutil.DevError(
+                "version string '%s' needs two '.' in it." % verstr)
 
     return ((int(verstr.split(".")[0]) * 1000000) +
             (int(verstr.split(".")[1]) * 1000) + (int(verstr.split(".")[2])))
@@ -113,8 +114,8 @@ class _SupportCheck(object):
         for vstr in versions:
             v = _version_str_to_int(vstr)
             if vstr is not None and v != 0 and v < 7003:
-                raise RuntimeError(  # pragma: no cover
-                    "programming error: Cannot enforce "
+                raise xmlutil.DevError(
+                    "Cannot enforce "
                     "support checks for libvirt versions less than 0.7.3, "
                     "since required APIs were not available. ver=%s" % vstr)
 
@@ -227,24 +228,16 @@ class SupportCache:
         self._cache = {}
         self._virtconn = virtconn
 
+    conn_domain = _make(
+        function="virConnect.listAllDomains", run_args=())
     conn_storage = _make(
-        function="virConnect.listStoragePools", run_args=())
+        function="virConnect.listAllStoragePools", run_args=())
     conn_nodedev = _make(
         function="virConnect.listDevices", run_args=(None, 0))
-    conn_network = _make(function="virConnect.listNetworks", run_args=())
-    conn_interface = _make(
-        function="virConnect.listInterfaces", run_args=())
+    conn_network = _make(
+        function="virConnect.listNetworks", run_args=())
+
     conn_stream = _make(function="virConnect.newStream", run_args=(0,))
-    conn_listalldomains = _make(
-        function="virConnect.listAllDomains", run_args=())
-    conn_listallnetworks = _make(
-        function="virConnect.listAllNetworks", run_args=())
-    conn_listallstoragepools = _make(
-        function="virConnect.listAllStoragePools", run_args=())
-    conn_listallinterfaces = _make(
-        function="virConnect.listAllInterfaces", run_args=())
-    conn_listalldevices = _make(
-        function="virConnect.listAllDevices", run_args=())
     conn_working_xen_events = _make(hv_version={"xen": "4.0.0", "all": 0})
     # This is an arbitrary check to say whether it's a good idea to
     # default to qcow2. It might be fine for xen or qemu older than the versions
@@ -261,20 +254,13 @@ class SupportCache:
     conn_domain_capabilities = _make(
         function="virConnect.getDomainCapabilities",
         run_args=(None, None, None, None))
-    conn_domain_reset = _make(version="0.9.7", hv_version={"qemu": 0})
     conn_vmport = _make(
         version="1.2.16", hv_version={"qemu": "2.2.0", "test": 0})
     conn_mem_stats_period = _make(
         function="virDomain.setMemoryStatsPeriod",
-        version="1.1.1", hv_version={"qemu": 0})
+        version="1.1.1", hv_version={"qemu": 0, "test": "5.6.0"})
     # spice GL is actually enabled with libvirt 1.3.3, but 3.1.0 is the
     # first version that sorts out the qemu:///system + cgroup issues
-    conn_spice_gl = _make(version="3.1.0",
-        hv_version={"qemu": "2.6.0", "test": 0})
-    conn_spice_rendernode = _make(version="3.1.0",
-        hv_version={"qemu": "2.9.0", "test": 0})
-    conn_video_virtio_accel3d = _make(version="1.3.0",
-        hv_version={"qemu": "2.5.0", "test": 0})
     conn_graphics_listen_none = _make(version="2.0.0")
     conn_rng_urandom = _make(version="1.3.4")
     conn_usb3_ports = _make(version="1.3.5")
@@ -314,16 +300,10 @@ class SupportCache:
         version="1.2.8", hv_version={"qemu": 0})
 
     # Pool checks
-    pool_isactive = _make(function="virStoragePool.isActive", run_args=())
-    pool_listallvolumes = _make(
-        function="virStoragePool.listAllVolumes", run_args=())
     pool_metadata_prealloc = _make(
         flag="VIR_STORAGE_VOL_CREATE_PREALLOC_METADATA",
         version="1.0.1")
 
-    # Network checks
-    net_isactive = _make(function="virNetwork.isActive", run_args=())
-
 
     def _check_version(self, version):
         """
